import L from 'leaflet';
import { mapViewModes, mapViewNames } from './constants';

var LayerManager = /*#__PURE__*/function () {
  function LayerManager(id) {
    this._currentLayer = 'normal.day';
    this.layers = {};
  }

  var _proto = LayerManager.prototype;

  _proto.createMapLayers = function createMapLayers(_ref) {
    var _this = this;

    var appCode = _ref.appCode,
        appId = _ref.appId,
        attribution = _ref.attribution,
        id = _ref.id,
        language = _ref.language,
        mapViewModes = _ref.mapViewModes,
        maxZoom = _ref.maxZoom,
        minZoom = _ref.minZoom;
    var tileLayers = [];
    mapViewModes.forEach(function (value, index) {
      var baseMapView = _this.getBaseMapView(value);

      var tileLayer = L.tileLayer('https://{s}.{base}.maps.api.here.com/maptile/2.1/maptile/{mapVersion}/' + value + '/{z}/{x}/{y}/256/png8?app_id={app_id}&app_code={app_code}&lg={language}', {
        app_code: appCode,
        app_id: appId,
        attribution: attribution,
        base: baseMapView,
        language: language,
        mapVersion: 'newest',
        maxZoom: maxZoom,
        minZoom: minZoom,
        subdomains: '1234'
      });
      tileLayers[index] = tileLayer;
    });
    this.layers.map = tileLayers;
  };

  _proto.getLayerGroup = function getLayerGroup(layer) {
    return L.featureGroup(layer);
  };

  _proto.addChangeViewController = function addChangeViewController(properties, map) {
    var controllers = {};
    this.layers.map.forEach(function (layer, index) {
      controllers[mapViewNames[index]] = layer;
    });
    properties.enableViewMenu && L.control.layers(controllers).addTo(map);
  };

  _proto.addLayersToGroup = function addLayersToGroup(layers, groupName) {
    var _this2 = this;

    !this.layers[groupName] && (this.layers[groupName] = this.getLayerGroup(layers));
    layers.map(function (layer) {
      _this2.layers[groupName].addLayer(layer);
    });
  };

  _proto.removeLayersFromGroup = function removeLayersFromGroup(layersToDelete, groupName) {
    var _this3 = this;

    !this.layers[groupName] && (this.layers.markers = this.getFullLayerGroup());
    this.layers[groupName].eachLayer(function (layer) {
      var found = layersToDelete.find(function (pointToDelete) {
        return layer.Id === pointToDelete.Id;
      });
      found && _this3.layers.markers.removeLayer(layer);
    });
  };

  _proto.getBaseMapView = function getBaseMapView(value) {
    return value === mapViewModes.SATELLITE ? 'aerial' : 'base';
  };

  _proto.addHeatMapLayer = function addHeatMapLayer(url, map) {
    this.layers.heatMap = L.tileLayer(url, {
      pane: 'overlayPane',
      opacity: 0.7,
      zIndex: 1
    });
    this.layers.heatMap.addTo(map);
  };

  _proto.getLayers = function getLayers(layer) {
    return layer ? layer.getLayers() : [];
  };

  return LayerManager;
}();

export { LayerManager as default };