import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import L from 'leaflet';
import Polygons from './shapes/Polygons';
import Circle from './shapes/Circle';
import { mapViewModes } from './constants';
import MarkerManager from './marker-manager';
import LayerManager from './layer-manager';
var DRAW_CLASS = 'drawnPolygon';
var simplifyGeoJson = null;

var LeafletMap = /*#__PURE__*/function () {
  function LeafletMap(properties) {
    this.setSimplifyDraw(properties);
    this.setMapDOMInstance(properties.mapDOMInstance);
    this.createMarkerManager(properties.mapDOMInstance);
    this.createLayerManager(properties.id);
    this.buildMap(properties);
    this.setZoomControlPosition(properties.zoomControlPosition);
    this.buildShapes(properties);
    this.subscribeToLeafletMapEvents();
    this.layerManager.addChangeViewController(properties, this._map, this._normalViewText, this._satelliteViewText);
    this.markerManager.addIconMarkersToMap({
      icons: properties.icons,
      map: this._map
    });
    this.preInitDrawing(properties);
    this.dispatchFirstLoad();
  }

  var _proto = LeafletMap.prototype;

  _proto.setViewCenter = function setViewCenter(coordinates, zoom) {
    this._map.setView(new L.LatLng(coordinates[0], coordinates[1]), zoom);
  };

  _proto.setSimplifyDraw = function setSimplifyDraw(_ref) {
    var simplifyDraw = _ref.simplifyDraw,
        simplifyDrawMinimumCoordinates = _ref.simplifyDrawMinimumCoordinates,
        simplifyTolerance = _ref.simplifyTolerance,
        simplifyHighQuality = _ref.simplifyHighQuality;
    this._simplifyDraw = simplifyDraw;
    this._simplifyDrawMinimumCoordinates = simplifyDrawMinimumCoordinates;
    this._simplifyTolerance = simplifyTolerance;
    this._simplifyHighQuality = simplifyHighQuality;
  };

  _proto.buildMap = function buildMap(properties) {
    this.layerManager.createMapLayers(properties);
    var mapOptions = {
      center: [properties.latitude, properties.longitude],
      dragging: properties.dragging,
      layers: [this.layerManager.layers.map[properties.selectedMapViewMode]],
      scrollWheelZoom: properties.scrollWheelZoom,
      zoom: properties.zoom,
      zoomControl: properties.zoomControl
    };
    this._map = L.map(properties.id, mapOptions);
    this.attachPropsToMapInstance(properties);
  };

  _proto.dispatchFirstLoad = function dispatchFirstLoad() {
    this.dispatchCustomEvent({
      eventName: 'leaflet_map_loaded',
      detail: this.getParamsForRequest()
    });
  };

  _proto.buildShapes = function buildShapes(properties) {
    properties.polygons && this.buildPolygons(_extends({}, properties));
    properties.radius && this.buildCircle(_extends({}, properties));
  };

  _proto.setMapDOMInstance = function setMapDOMInstance(mapDOMInstance) {
    this.mapDOM = mapDOMInstance;
  };

  _proto.buildPolygons = function buildPolygons(_ref2) {
    var currentGeoCode = _ref2.currentGeoCode,
        hoverStyles = _ref2.hoverStyles,
        onLayerClick = _ref2.onLayerClick,
        onPolygonWithBounds = _ref2.onPolygonWithBounds,
        polygons = _ref2.polygons,
        radius = _ref2.radius,
        showLabels = _ref2.showLabels;
    this.polygons = new Polygons({
      currentGeoCode: currentGeoCode,
      hoverStyles: hoverStyles,
      onLayerClick: onLayerClick,
      onPolygonWithBounds: onPolygonWithBounds,
      radius: radius,
      showLabels: showLabels
    });
    this.polygons.setPolygonsOnMap({
      map: this._map,
      polygons: polygons
    });
  };

  _proto.buildCircle = function buildCircle(_ref3) {
    var latitude = _ref3.latitude,
        longitude = _ref3.longitude,
        radius = _ref3.radius,
        onPolygonWithBounds = _ref3.onPolygonWithBounds;
    this.circle = new Circle({
      latitude: latitude,
      longitude: longitude,
      radius: radius,
      onPolygonWithBounds: onPolygonWithBounds
    });
    this.circle.setCircleOnMap({
      map: this._map
    });
  };

  _proto.createMarkerManager = function createMarkerManager(mapId) {
    this.markerManager = new MarkerManager(mapId);
  };

  _proto.createLayerManager = function createLayerManager() {
    this.layerManager = new LayerManager();
  };

  _proto.showHeatMap = function showHeatMap(url) {
    this.layerManager.addHeatMapLayer(url, this._map);
  };

  _proto.removeHeatMapLayer = function removeHeatMapLayer() {
    this._map.removeLayer(this.layerManager.layers.heatMap);
  };

  _proto.setZoomControlPosition = function setZoomControlPosition(position) {
    this._map.zoomControl && this._map.zoomControl.setPosition(position);
  };

  _proto.attachPropsToMapInstance = function attachPropsToMapInstance(properties) {
    this._map.props = properties;
  };

  _proto.setCenter = function setCenter(options) {
    this.setCenter(L.latLng(parseFloat(options.latitude), parseFloat(options.longitude)));
    options.zoom && this.setZoom(options.zoom);
  };

  _proto.getBounds = function getBounds() {
    return {
      northEast: this._map.getBounds().getNorthEast(),
      northWest: this._map.getBounds().getNorthWest(),
      southEast: this._map.getBounds().getSouthEast(),
      southWest: this._map.getBounds().getSouthWest()
    };
  };

  _proto.getZoomLevel = function getZoomLevel() {
    return this._map.getZoom();
  };

  _proto.getCenter = function getCenter() {
    return {
      latitude: this._map.getCenter().lat,
      longitude: this._map.getCenter().lng
    };
  };

  _proto.getCenterWithZoom = function getCenterWithZoom() {
    return _extends({}, this.getCenter(), {
      zoom: this.getZoomLevel()
    });
  };

  _proto.clearLayer = function clearLayer(layer) {
    layer.clearLayers();

    this._map.removeLayer(layer);
  };

  _proto.addLayerGroupToMap = function addLayerGroupToMap(layer) {
    this._map.addLayer(layer);
  };

  _proto.invalidateSize = function invalidateSize(effect) {
    this._map.invalidateSize(effect);
  };

  _proto.setView = function setView(viewType) {
    if (viewType === mapViewModes.NORMAL && this._currentLayer !== mapViewModes.NORMAL) {
      this._map.removeLayer(this.layerManager.layers.map[1]);

      this._map.addLayer(this.layerManager.layers.map[0]);

      this._currentLayer = viewType;
    } else if (viewType === mapViewModes.SATELLITE && this._currentLayer !== mapViewModes.SATELLITE) {
      this._map.removeLayer(this.layerManager.layers.map[0]);

      this._map.addLayer(this.layerManager.layers.map[1]);

      this._currentLayer = viewType;
    }
  };

  _proto.subscribeToLeafletMapEvents = function subscribeToLeafletMapEvents() {
    var _this = this;

    this._map.on('load', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_load',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('click', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_click',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('drag', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_drag',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('dragend', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_dragend',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('zoomend', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_zoomend',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('baselayerchange', function (e) {
      if (e.name === _this._tileLayerNormal) {
        _this.dispatchCustomEvent({
          eventName: 'leaflet_map_layer_normal',
          detail: _this.getParamsForRequest()
        });
      }
    });

    this._map.on('baselayerchange', function (e) {
      if (e.name === _this._tileLayerSatName) {
        _this.dispatchCustomEvent({
          eventName: 'leaflet_map_layer_satellite',
          detail: _this.getParamsForRequest()
        });
      }
    });
  };

  _proto.getMapBoundingBox = function getMapBoundingBox() {
    var _this$getBounds = this.getBounds(),
        northWest = _this$getBounds.northWest,
        southEast = _this$getBounds.southEast;

    return northWest.lng + ',' + northWest.lat + ';' + northWest.lng + ',' + southEast.lat + ';' + southEast.lng + ',' + southEast.lat + ';' + southEast.lng + ',' + northWest.lat + ';' + northWest.lng + ',' + northWest.lat;
  };

  _proto.getParamsForRequest = function getParamsForRequest() {
    var _this$getCenterWithZo = this.getCenterWithZoom(this._map),
        zoom = _this$getCenterWithZo.zoom,
        latitude = _this$getCenterWithZo.latitude,
        longitude = _this$getCenterWithZo.longitude;

    return {
      latitude: latitude,
      longitude: longitude,
      mapBoundingBox: this.getMapBoundingBox(),
      zoomLevel: zoom
    };
  };

  _proto.clearMarkersLayer = function clearMarkersLayer() {
    this.layerManager.layers.markers && this.clearLayer(this.layerManager.layers.markers);
  } // Returns the elements in arrayToCompare that are not present in the arrayOrigin
  ;

  _proto.getPositiveDiffOfArraysOfPoints = function getPositiveDiffOfArraysOfPoints(_ref4) {
    var pois = _ref4.pois,
        currentMarkers = _ref4.currentMarkers;
    return pois.reduce(function (accumulate, originalArrayPoint) {
      !currentMarkers.some(function (comparedArrayPoint) {
        return comparedArrayPoint.Id === originalArrayPoint.Id && comparedArrayPoint.options.icon.options.className.includes(originalArrayPoint.customClassName);
      }) && accumulate.push(originalArrayPoint);
      return accumulate;
    }, []);
  };

  _proto.getPositiveDiffOfArraysOfPointsAux = function getPositiveDiffOfArraysOfPointsAux(_ref5) {
    var pois = _ref5.pois,
        currentMarkers = _ref5.currentMarkers;
    return currentMarkers.reduce(function (accumulate, originalArrayPoint) {
      !pois.some(function (comparedArrayPoint) {
        return comparedArrayPoint.Id === originalArrayPoint.Id && originalArrayPoint.options.icon.options.className.includes(comparedArrayPoint.customClassName);
      }) && accumulate.push(originalArrayPoint);
      return accumulate;
    }, []);
  };

  _proto.dispatchCustomEvent = function dispatchCustomEvent(_ref6) {
    var eventName = _ref6.eventName,
        detail = _ref6.detail;
    var event;

    if (this.mapDOM.CustomEvent && typeof this.mapDOM.CustomEvent === 'function') {
      event = new this.mapDOM.CustomEvent(eventName, {
        detail: detail
      });
    } else {
      event = document.createEvent('CustomEvent');
      event.initCustomEvent(eventName, true, true, detail);
    }

    this.mapDOM.dispatchEvent(event);
  };

  _proto.addLayersToMap = function addLayersToMap(layerDataArray, groupName) {
    var _this2 = this;

    var layers = layerDataArray.map(function (layerData) {
      return _this2.markerManager.createMarker(layerData);
    });
    this.layerManager.addLayersToGroup(layers, groupName);
    this.addLayerGroupToMap(this.layerManager.layers[groupName]);
  } // We should check if there's a better solution for cluster poi's like the use of plugins http://leafletjs.com/plugins.html#clusteringdecluttering
  ;

  _proto.displayPois = function displayPois(pois) {
    if (!pois || !pois.length) {
      this.clearMarkersLayer();
    } else {
      // Get new Marker Type by checking one new POI.
      var markerType = pois[0].markerType;
      var shouldClearAllMarkers = markerType !== this.markerManager.currentMarkerType; // If we render a new type of Marker, then we need to clear all Markers.

      shouldClearAllMarkers && this.clearMarkersLayer(); // Get current array of map Markers.

      var currentMarkers = this.layerManager.getLayers(this.layerManager.layers.markers); // To prevent repaint all POIs that are already visible, get the news to Add and the ones to be Removed.

      var pointsToAdd = this.getPositiveDiffOfArraysOfPoints({
        pois: pois,
        currentMarkers: currentMarkers
      });
      var pointsToDelete = this.getPositiveDiffOfArraysOfPointsAux({
        currentMarkers: currentMarkers,
        pois: pois
      });
      var pointsAreClickable = markerType > 0; // Keep the current lat/lng of a POI that is going to be repaint due to selection with a new random lat/lng.
      // In order to prevent that POIs change its lat/lon for de-clusterized properties.
      // markerType > 0 means that POIs are clickable, so they can be selected, and for instance, repaint.

      pointsToDelete.length && pointsAreClickable && pointsToAdd.forEach(function (pointToAdd) {
        var pointToRefresh = pointsToDelete.find(function (pointToDelete) {
          return pointToDelete.Id === pointToAdd.Id;
        });

        if (pointToRefresh) {
          pointToAdd.latitude = pointToRefresh._latlng.lat;
          pointToAdd.longitude = pointToRefresh._latlng.lng;
        }
      }); // Set new markerType (0, 1, 2).

      this.markerManager.resetMarkerType(markerType);
      this.markerManager.currentMarkerType = markerType; // Remove pointsToDelete, and Add new POI's to map.

      pointsToDelete.length && this.layerManager.removeLayersFromGroup(pointsToDelete, 'markers');
      pointsToAdd.length && this.addLayersToMap(pointsToAdd, 'markers');
    }
  };

  _proto.preInitDrawing = function preInitDrawing(properties) {
    var _this3 = this;

    // For the drawing-in-progress controller object
    this._drawingPolygon = null; // For the already finished Geo JSON drawn shape

    this._drawnPolygonRemoveButton = null;
    this._drawnPolygon = L.geoJson(properties.initialDrawnPolygon, {
      className: DRAW_CLASS
    }).getLayers()[0] || null; // Save drawing events

    this._drawingEvents = {
      onDrawPolygonStop: properties.onDrawPolygonStop,
      onDrawPolygonFinish: properties.onDrawPolygonFinish,
      onDrawPolygonRemove: properties.onDrawPolygonRemove
    }; // Add an existing drawn polygon if any

    if (this._drawnPolygon) {
      this.initAsyncDrawingLayer().then(function () {
        var _this3$_drawnPolygon;

        // Fit and center map view to drawn polygon
        var bounds = (_this3$_drawnPolygon = _this3._drawnPolygon) == null ? void 0 : _this3$_drawnPolygon.getBounds();
        if (bounds != null && bounds.isValid()) _this3._map.fitBounds(bounds, {
          animate: false
        }); // Add it to the map

        _this3.drawingAddFinishedPolygon(_this3._drawnPolygon, false);
      });
    }
  };

  _proto.initAsyncDrawingLayer = /*#__PURE__*/function () {
    var _initAsyncDrawingLayer = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var _this4 = this;

      var _yield$Promise$all, simplifyModule, drawnItems;

      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!(L.Draw && this._drawControl)) {
                _context.next = 2;
                break;
              }

              return _context.abrupt("return");

            case 2:
              _context.next = 4;
              return Promise.all([import('@turf/simplify'), import('leaflet-draw')]);

            case 4:
              _yield$Promise$all = _context.sent;
              simplifyModule = _yield$Promise$all[0];
              simplifyGeoJson = simplifyModule["default"]; // Create editable layer

              drawnItems = new L.FeatureGroup();

              this._map.addLayer(drawnItems); // Crate controller


              this._drawControl = new L.Control.Draw({
                edit: {
                  featureGroup: drawnItems
                }
              });

              this._map.addControl(this._drawControl);
              /**
               * Add drawing events
               */


              this._map.on(L.Draw.Event.CREATED, function (_ref7) {
                var drawnPolygon = _ref7.layer;

                _this4.drawingAddFinishedPolygon(drawnPolygon);
              });

              this._map.on(L.Draw.Event.DRAWSTOP, function () {
                _this4._drawingEvents.onDrawPolygonStop();
              });

            case 13:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function initAsyncDrawingLayer() {
      return _initAsyncDrawingLayer.apply(this, arguments);
    }

    return initAsyncDrawingLayer;
  }();

  _proto.drawingCheckPlugin = function drawingCheckPlugin() {
    if (!L.Draw) throw new Error('`leaflet-draw` is still not loaded!');
  };

  _proto.drawingStartPolygon = function drawingStartPolygon() {
    var polygonOption = this._drawControl.options.polygon;
    this._drawingPolygon = new L.Draw.Polygon(this._map, polygonOption);

    this._drawingPolygon.setOptions({
      shapeOptions: {
        className: DRAW_CLASS
      }
    });

    this._drawingPolygon.enable();
  };

  _proto.drawingStopPolygon = function drawingStopPolygon() {
    if (!this._drawingPolygon) return -1;

    this._drawingPolygon.disable();

    this._drawingPolygon = null;
  };

  _proto.drawingAddFinishedPolygon = function drawingAddFinishedPolygon(drawnPolygon, triggerEvent) {
    var _this5 = this;

    if (triggerEvent === void 0) {
      triggerEvent = true;
    }

    // add polygon to map
    var shouldSimplifyDraw = this._simplifyDraw && drawnPolygon._latlngs[0].length >= this._simplifyDrawMinimumCoordinates;
    var simplifiedPolygon = shouldSimplifyDraw && L.geoJson(simplifyGeoJson(drawnPolygon.toGeoJSON(), {
      tolerance: this._simplifyTolerance,
      highQuality: this._simplifyHighQuality
    }), {
      className: DRAW_CLASS
    }).getLayers()[0] || null;
    this._drawnPolygon = shouldSimplifyDraw ? simplifiedPolygon : drawnPolygon;

    this._map.addLayer(this._drawnPolygon); // add the 'remove' button


    var icon = L.divIcon({
      className: DRAW_CLASS + "RemoveButton",
      iconSize: [40, 40]
    });
    var coords = this._drawnPolygon._latlngs[0];
    var buttonCoord = coords.reduce(function (acc, coord) {
      return coord.lat > acc.lat ? coord : acc;
    }, coords[0]);
    this._drawnPolygonRemoveButton = L.marker([buttonCoord.lat, buttonCoord.lng], {
      icon: icon
    }).on('click', function () {
      return _this5.drawingClear();
    }).addTo(this._map);

    if (triggerEvent) {
      this._drawingEvents.onDrawPolygonFinish(this._drawnPolygon.toGeoJSON());
    }
  };

  _proto.drawingClear = function drawingClear() {
    if (this._drawnPolygon) {
      this._map.removeLayer(this._drawnPolygon);

      this._map.removeLayer(this._drawnPolygonRemoveButton);

      this._drawingEvents.onDrawPolygonRemove({
        polygon: this._drawnPolygon.toGeoJSON(),
        mapBoundingBox: this.getMapBoundingBox()
      });

      this._drawnPolygon = null;
      this._drawnPolygonRemoveButton = null;
    }
  };

  _proto.getPublicAPI = function getPublicAPI() {
    var _this6 = this;

    return {
      zoomIn: function zoomIn() {
        return _this6._map.zoomIn();
      },
      zoomOut: function zoomOut() {
        return _this6._map.zoomOut();
      },
      drawing: {
        load: function load() {
          return _this6.initAsyncDrawingLayer();
        },
        startPolygon: function startPolygon() {
          _this6.drawingCheckPlugin();

          _this6.drawingClear();

          _this6.drawingStartPolygon();
        },
        stopPolygon: function stopPolygon() {
          _this6.drawingCheckPlugin();

          _this6.drawingStopPolygon();
        },
        clear: function clear() {
          _this6.drawingCheckPlugin();

          _this6.drawingClear();
        }
      }
    };
  };

  return LeafletMap;
}();

export { LeafletMap as default };