import L from 'leaflet';

var SearchMapPolygons = /*#__PURE__*/function () {
  function SearchMapPolygons(_ref) {
    var currentGeoCode = _ref.currentGeoCode,
        hoverStyles = _ref.hoverStyles,
        onLayerClick = _ref.onLayerClick,
        onPolygonWithBounds = _ref.onPolygonWithBounds,
        radius = _ref.radius,
        showLabels = _ref.showLabels;
    this._polygonList = [];
    this._storedPolygons = {};
    this.SPAIN_POLYGON_CENTER = {
      latitude: 40.4096,
      longitude: -3.68624
    };
    this.SPAIN_POLYGON_NAME = 'geom_724_0_0_0_0_0_0_0_0';
    this.BASE_CLASSNAME = 'scm-map__area';
    this.HOVER_CLASSNAME = 'is-hover';
    this.currentGeoCode = currentGeoCode;
    this.hoverStyles = hoverStyles;
    this.onLayerClick = onLayerClick;
    this.onPolygonWithBounds = onPolygonWithBounds;
    this.radius = radius;
    this.showLabels = showLabels;
  }

  var _proto = SearchMapPolygons.prototype;

  _proto.removePolygonsFromMap = function removePolygonsFromMap(map) {
    if (this._polygonList.length > 0) {
      this._polygonList.forEach(function (polygon) {
        return map.removeLayer(polygon);
      });
    }
  };

  SearchMapPolygons.zoomIn = function zoomIn(evt) {
    var map = evt.target._map;
    map.setView(evt.latlng, map.getZoom() + 2);
  };

  _proto.printPolygonOnMap = function printPolygonOnMap(_ref2) {
    var fitBound = _ref2.fitBound,
        map = _ref2.map,
        polygon = _ref2.polygon;
    var BASE_CLASSNAME = this.BASE_CLASSNAME,
        hoverStyles = this.hoverStyles;
    var className = fitBound && !this.radius ? BASE_CLASSNAME + " fitBound" : BASE_CLASSNAME;
    var that = this;

    var getTooltipElement = function getTooltipElement(feature) {
      var _feature$getTooltip;

      return (_feature$getTooltip = feature.getTooltip()) == null ? void 0 : _feature$getTooltip.getElement();
    };

    var polygonGeoJSon = L.geoJson(polygon, {
      className: className,
      onEachFeature: function onEachFeature(feature, layer) {
        layer.on({
          mouseout: function mouseout(e) {
            var originalEvent = e.originalEvent; // get the element we're entering now

            var elementAfterOut = document.elementFromPoint(originalEvent.clientX, originalEvent.clientY);

            if (elementAfterOut) {
              // if the element is a marker, avoid removing the hover class
              if (!elementAfterOut.classList.contains('leaflet-marker-icon') && elementAfterOut !== originalEvent.target) {
                originalEvent.target.classList.remove(that.HOVER_CLASSNAME);
              }
            }

            hoverStyles && polygonGeoJSon.resetStyle(this);
            var tooltipElement = getTooltipElement(this);
            if (tooltipElement) tooltipElement.classList.remove(that.HOVER_CLASSNAME);
          },
          mouseover: function mouseover(e) {
            hoverStyles && this.setStyle(hoverStyles); // remove previous hovered polygons

            var polygonWithHover = document.querySelector("." + that.HOVER_CLASSNAME); // if there's a previous polygon, remove the hover class

            polygonWithHover && polygonWithHover.classList.remove(that.HOVER_CLASSNAME); // add the hover classname to the new polygon

            e.originalEvent.target.classList.add(that.HOVER_CLASSNAME);
            var tooltipElement = getTooltipElement(this);
            if (tooltipElement) tooltipElement.classList.add(that.HOVER_CLASSNAME);

            if (!L.Browser.ie && !L.Browser.opera) {
              this.bringToFront();
            }
          },
          click: function click(event) {
            var _layer$feature, _layer$feature$proper;

            if (!(layer != null && (_layer$feature = layer.feature) != null && (_layer$feature$proper = _layer$feature.properties) != null && _layer$feature$proper.Code)) return;
            var Code = layer.feature.properties.Code;
            return !that.currentGeoCode.includes(Code) ? that.onLayerClick(event) : function () {};
          }
        });
      }
    });

    this._polygonList.push(polygonGeoJSon);

    polygonGeoJSon.addTo(map);

    if (this.showLabels) {
      polygonGeoJSon.eachLayer(function (layer) {
        var _layer$feature2, _layer$feature2$prope, _layer$feature3, _layer$feature3$prope;

        if (!(layer != null && (_layer$feature2 = layer.feature) != null && (_layer$feature2$prope = _layer$feature2.properties) != null && _layer$feature2$prope.LocationName) || !(layer != null && (_layer$feature3 = layer.feature) != null && (_layer$feature3$prope = _layer$feature3.properties) != null && _layer$feature3$prope.Code)) return;

        try {
          var _layer$feature$proper2 = layer.feature.properties,
              Code = _layer$feature$proper2.Code,
              LocationName = _layer$feature$proper2.LocationName,
              _layer$feature$proper3 = _layer$feature$proper2.counter,
              counter = _layer$feature$proper3 === void 0 ? '' : _layer$feature$proper3;
          var tooltipLabelBlock = "<span class=\"leaflet-tooltip-label\">" + LocationName + "</span>";
          var tooltipCounterBlock = "<span class=\"leaflet-tooltip-counter\">" + counter + "</span>";
          var allTooltipLabel = counter ? "" + tooltipLabelBlock + tooltipCounterBlock : tooltipLabelBlock;
          !that.currentGeoCode.includes(Code) && layer.bindTooltip(allTooltipLabel, {
            permanent: true,
            direction: 'center',
            className: fitBound ? 'is-fit-bound' : ''
          }).openTooltip();
        } catch (error) {}
      });
    }

    var polygonName = polygon.properties && polygon.properties.Code;

    if (polygonName !== this.SPAIN_POLYGON_NAME) {
      var bounds = polygonGeoJSon.getBounds();

      if (bounds.isValid()) {
        return bounds;
      }
    } else {
      var _this$SPAIN_POLYGON_C = this.SPAIN_POLYGON_CENTER,
          latitude = _this$SPAIN_POLYGON_C.latitude,
          longitude = _this$SPAIN_POLYGON_C.longitude;
      var centerSpain = new L.LatLng(latitude, longitude);
      map.panTo(centerSpain);
    }
  };

  _proto.setPolygonsOnMap = function setPolygonsOnMap(_ref3) {
    var _this = this;

    var map = _ref3.map,
        polygons = _ref3.polygons;
    var bounds;
    this.removePolygonsFromMap(map);

    if (!(polygons instanceof Array)) {
      polygons = [polygons];
    }

    polygons.forEach(function (polygon) {
      var _polygon$fitBound = polygon.fitBound,
          fitBound = _polygon$fitBound === void 0 ? true : _polygon$fitBound;

      if (fitBound) {
        bounds = bounds ? bounds.extend(_this.printPolygonOnMap({
          fitBound: fitBound,
          map: map,
          polygon: polygon
        })) : _this.printPolygonOnMap({
          fitBound: fitBound,
          map: map,
          polygon: polygon
        });
      } else {
        _this.printPolygonOnMap({
          fitBound: fitBound,
          map: map,
          polygon: polygon
        });
      }
    });
    this.onPolygonWithBounds({
      bounds: bounds,
      map: map
    });
    return true;
  };

  return SearchMapPolygons;
}();

export { SearchMapPolygons as default };